#cython: language_level=3
 
from .capi.obidmscolumn cimport obi_close_column,\
                                obi_truncate_and_close_column, \
                                obi_column_get_obibool_with_elt_name, \
                                obi_column_get_obibool_with_elt_idx, \
                                obi_column_set_obibool_with_elt_name, \
                                obi_column_set_obibool_with_elt_idx
from .capi.obierrno     cimport obi_errno
from .capi.obitypes     cimport OBIBool_NA

from obitools3.utils cimport str2bytes

from cpython.bool cimport PyBool_FromLong


cdef class OBIDMS_column_bool(OBIDMS_column):

    cpdef object get_line(self, index_t line_nb):
        cdef obibool_t value
        cdef object result
        value = obi_column_get_obibool_with_elt_idx(self.pointer, line_nb, 0)
        if obi_errno > 0 :
            raise IndexError(line_nb)
        if value == OBIBool_NA :
            result = None
        else :
            result = PyBool_FromLong(value)
        return result

    cpdef set_line(self, index_t line_nb, object value):
        raise Exception("Column is read-only")
    
    cpdef close(self):
        if obi_close_column(self.pointer) < 0 :
            raise Exception("Problem closing a column")


cdef class OBIDMS_column_bool_writable(OBIDMS_column_bool):     

    cpdef set_line(self, index_t line_nb, object value):
        if obi_column_set_obibool_with_elt_idx(self.pointer, line_nb, 0, <obibool_t> value) < 0:
            raise Exception("Problem setting a value in a column")

    cpdef close(self):
        if obi_truncate_and_close_column(self.pointer) < 0 :
            raise Exception("Problem closing a column")


cdef class OBIDMS_column_bool_multi_elts(OBIDMS_column_bool):

    cpdef object get_item(self, index_t line_nb, str element_name):
        cdef obibool_t value
        cdef object result
        value = obi_column_get_obibool_with_elt_name(self.pointer, line_nb, str2bytes(element_name))
        if obi_errno > 0 :
            raise IndexError(line_nb, element_name)
        if value == OBIBool_NA :
            result = None
        else :
            result = PyBool_FromLong(value)
        return result
    
    cpdef object get_line(self, index_t line_nb) :
        cdef obibool_t value
        cdef object result
        cdef index_t i
        cdef bint all_NA
        result = {}
        all_NA = True
        for i in range(self.nb_elements_per_line) :
            value = obi_column_get_obibool_with_elt_idx(self.pointer, line_nb, i)
            if obi_errno > 0 :
                raise IndexError(line_nb)           
            result[self.elements_names[i]] = PyBool_FromLong(value)
            if all_NA and (value != OBIBool_NA) :
                all_NA = False
        if all_NA :
            result = None
        return result

    cpdef set_item(self, index_t line_nb, str element_name, obibool_t value):
        raise Exception("Column is read-only")

    cpdef set_line(self, index_t line_nb, object values):
        raise Exception("Column is read-only")


cdef class OBIDMS_column_bool_multi_elts_writable(OBIDMS_column_bool_multi_elts):     

    cpdef set_item(self, index_t line_nb, str element_name, obibool_t value):
        if obi_column_set_obibool_with_elt_name(self.pointer, line_nb, str2bytes(element_name), value) < 0:
            raise Exception("Problem setting a value in a column")
    
    cpdef set_line(self, index_t line_nb, object values):
        cdef obibool_t value
        for element_name in values :
            value = <obibool_t> values[element_name]
            self.set_item(line_nb, element_name, value)

    cpdef close(self):
        if obi_truncate_and_close_column(self.pointer) < 0 :
            raise Exception("Problem closing a column")

