#cython: language_level=3

from ..capi.obidms   cimport OBIDMS_p
from ..capi.obitypes cimport const_char_p, \
                             OBIType_t, \
                             obiversion_t, \
                             obiint_t, \
                             obibool_t, \
                             obichar_t, \
                             obifloat_t, \
                             index_t, \
                             time_t


cdef extern from "obidmscolumn.h" nogil:
    
    struct OBIDMS_column_header_t:
        bint            little_endian
        int             header_size
        index_t         line_count
        index_t         lines_used
        index_t         nb_elements_per_line
        const_char_p    elements_names
        OBIType_t       data_type
        time_t          creation_date
        obiversion_t    version
        obiversion_t    cloned_from
        const_char_p    name
        const_char_p    array_name
        const_char_p    comments
    
    ctypedef OBIDMS_column_header_t* OBIDMS_column_header_p

    struct OBIDMS_column_t:
        OBIDMS_column_header_p header

    ctypedef OBIDMS_column_t* OBIDMS_column_p

    OBIDMS_column_p obi_create_column(OBIDMS_p dms, 
                                      const_char_p column_name, 
                                      OBIType_t type, 
                                      index_t nb_lines, 
                                      index_t nb_elements_per_line, 
                                      const_char_p elements_names,
                                      const_char_p array_name)
                
    OBIDMS_column_p obi_open_column(OBIDMS_p dms, 
                                    const_char_p column_name, 
                                    obiversion_t version_number)
        
    int obi_close_column(OBIDMS_column_p column)
        
    OBIDMS_column_p obi_clone_column(OBIDMS_p dms, 
                                     const_char_p column_name, 
                                     obiversion_t version_number, 
                                     bint clone_data)
    
    int obi_truncate_and_close_column(OBIDMS_column_p column)
    
    obiversion_t obi_column_get_latest_version_from_name(OBIDMS_p dms, 
                                                         const_char_p column_name)
        
    OBIDMS_column_header_p obi_column_get_header_from_name(OBIDMS_p dms, 
                                                           const_char_p column_name)
    
    int obi_unmap_header(OBIDMS_column_header_p header)
        
    char* obi_column_format_date(time_t date)

    
cdef extern from "obidmscolumn_int.h" nogil:
    int obi_column_set_obiint_with_elt_name(OBIDMS_column_p column, 
                                            index_t line_nb, 
                                            const_char_p element_name, 
                                            obiint_t value)

    int obi_column_set_obiint_with_elt_idx(OBIDMS_column_p column, 
                                           index_t line_nb, 
                                           index_t element_idx, 
                                        obiint_t value)
    
    obiint_t obi_column_get_obiint_with_elt_name(OBIDMS_column_p column, 
                                                 index_t line_nb, 
                                                 const_char_p element_name)
    
    obiint_t obi_column_get_obiint_with_elt_idx(OBIDMS_column_p column, 
                                                index_t line_nb, 
                                                index_t element_idx)


cdef extern from "obidmscolumn_bool.h" nogil:

    int obi_column_set_obibool_with_elt_name(OBIDMS_column_p column, 
                                             index_t line_nb, 
                                             const_char_p element_name, 
                                             obibool_t value)
    
    int obi_column_set_obibool_with_elt_idx(OBIDMS_column_p column, 
                                            index_t line_nb, 
                                            index_t element_idx, 
                                            obibool_t value)
    
    obibool_t obi_column_get_obibool_with_elt_name(OBIDMS_column_p column, 
                                                   index_t line_nb, 
                                                   const_char_p element_name)
    
    obibool_t obi_column_get_obibool_with_elt_idx(OBIDMS_column_p column, 
                                                  index_t line_nb, 
                                                  index_t element_idx)

cdef extern from "obidmscolumn_char.h" nogil:

    int obi_column_set_obichar_with_elt_name(OBIDMS_column_p column, 
                                             index_t line_nb, 
                                             const_char_p element_name, 
                                             obichar_t value)
    
    int obi_column_set_obichar_with_elt_idx(OBIDMS_column_p column, 
                                            index_t line_nb, 
                                            index_t element_idx, 
                                            obichar_t value)
    
    obichar_t obi_column_get_obichar_with_elt_name(OBIDMS_column_p column, 
                                                   index_t line_nb, 
                                                   const_char_p element_name)
    
    obichar_t obi_column_get_obichar_with_elt_idx(OBIDMS_column_p column, 
                                                  index_t line_nb, 
                                                  index_t element_idx)

cdef extern from "obidmscolumn_float.h" nogil:

    int obi_column_set_obifloat_with_elt_name(OBIDMS_column_p column, 
                                              index_t line_nb, 
                                              const_char_p element_name, 
                                              obifloat_t value)
    
    int obi_column_set_obifloat_with_elt_idx(OBIDMS_column_p column, 
                                             index_t line_nb, 
                                             index_t element_idx, 
                                             obifloat_t value)
    
    obifloat_t obi_column_get_obifloat_with_elt_name(OBIDMS_column_p column, 
                                                     index_t line_nb, 
                                                     const_char_p element_name)
    
    obifloat_t obi_column_get_obifloat_with_elt_idx(OBIDMS_column_p column, 
                                                    index_t line_nb, 
                                                    index_t element_idx)

cdef extern from "obidmscolumn_str.h" nogil:

    int obi_column_set_obistr_with_elt_name(OBIDMS_column_p column, 
                                            index_t line_nb, 
                                            const_char_p element_name, 
                                            char* value)
    
    int obi_column_set_obistr_with_elt_idx(OBIDMS_column_p column, 
                                           index_t line_nb, 
                                           index_t element_idx, 
                                           char* value)
    
    const_char_p obi_column_get_obistr_with_elt_name(OBIDMS_column_p column, 
                                              index_t line_nb, 
                                              const_char_p element_name)
    
    const_char_p obi_column_get_obistr_with_elt_idx(OBIDMS_column_p column, 
                                             index_t line_nb, 
                                             index_t element_idx)

